-- ****************************************************************************
-- Company:         CAEN SpA - Viareggio - Italy
-- Model:           V1495 -  Multipurpose Programmable Trigger Unit
-- FPGA Proj. Name: V1495USR_DEMO
-- Device:          ALTERA EP1C4F400C6
-- Author:          Luca Colombini (l.colombini@caen.it)
-- Date:            02-03-2006
-- ----------------------------------------------------------------------------
-- Module:          COIN_REFERENCE
-- Description:     Reference design to use the V1495 board 
--                  as a Coincidence Unit & I/O Register. 
--                  A gate pulse is generated on G port when a data 
--                  patterns on input ports A and B satisfy a trigger condition.
--                  The trigger condition implemented in this reference design
--                  is true when a bit-per-bit logic operation on port A and B 
--                  is true. The logic operator applied to Port A and B is
--                  selectable by means of a register bit (MODE Register Bit 4).
--                  If MODE bit 4 is set to '0', an AND logic operation is applied
--                  to corresponding bits in Port A and B.
--                  (i.e. A(0) AND B(0), A(1) AND B(1) etc.). 
--                  In this case, a trigger is generated if corresponding A and B
--                  port bits are '1' at the same time.
--                  If MODE bit 4 is set to '1', an OR logic operation is applied
--                  to corresponding bits in Port A and B.
--                  (i.e. A(0) OR B(0), A(1) OR B(1) etc.)
--                  In this case, a trigger is generated if there is a '1' on one
--                  bit of either port A or B.
--                  Port A and B bits can be singularly masked through a register,
--                  so that a '1' on that bit doesn't generate any trigger.
--                  Expansion mezzanine cards can be directly controlled through
--                  registers already implemented in this design.
--                  The expansion mezzanine is identified by a unique 
--                  identification code that can be read through a register.
-- ****************************************************************************

-- ############################################################################
-- Revision History:
--   Date         Author          Revision             Comments
--   02 Mar 06    LC              1.0                  Creation
--   20 Apr 09    EC              1.01                 Initial custom design
-- ############################################################################

-- ****************************************************************************
-- Company:         The Ohio State University - Columbus - Ohio - USA
-- Model:           V1495 -  Multipurpose Programmable Trigger Unit
-- FPGA Proj. Name: V1495USR_DEMO
-- Device:          ALTERA EP1C4F400C6
-- Author:          Eric Cochran (ecochran@mps.ohio-state.edu)
-- Date:            20-04-2009
-- ----------------------------------------------------------------------------
-- Module:          COIN_REFERENCE (Custom)
-- Description:     Modification of the original coin reference design to
--                  implement multiple coincidences and store timing
--                  information.
--                  Custom coincidence logic and timing currently implemented
--                  for channels:
--                    A0,B0
-- ****************************************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_misc.all;  -- Use OR_REDUCE function

USE work.v1495pkg.all;

ENTITY coin_reference IS
   PORT( 
      nLBRES      : IN     std_logic;                       -- Async Reset (active low)
      LCLK        : IN     std_logic;                       -- Local Bus Clock
      PLL_CLK     : IN     std_logic;                       -- PLL of Local Bus Clock
      PLL_LOCKED  : IN     std_logic;
      PLL_RESET   : OUT    std_logic;
      --*************************************************
      -- REGISTER INTERFACE
      --*************************************************
      REG_WREN    : IN     std_logic;                       -- Write pulse (active high)
      REG_RDEN    : IN     std_logic;                       -- Read  pulse (active high)
      REG_ADDR    : IN     std_logic_vector (15 DOWNTO 0);  -- Register address
      REG_DIN     : IN     std_logic_vector (15 DOWNTO 0);  -- Data from CAEN Local Bus
      REG_DOUT    : OUT    std_logic_vector (15 DOWNTO 0);  -- Data to   CAEN Local Bus
      USR_ACCESS  : IN     std_logic;                       -- Current register access is
                                                            -- at user address space(Active high)
      --*************************************************
      -- V1495 Front Panel Ports (PORT A,B,C,G)
      --*************************************************
      A_DIN       : IN     std_logic_vector (31 DOWNTO 0);  -- In A (32 x LVDS/ECL)
      B_DIN       : IN     std_logic_vector (31 DOWNTO 0);  -- In B (32 x LVDS/ECL)
      C_DOUT      : OUT    std_logic_vector (31 DOWNTO 0);  -- Out C (32 x LVDS)
      G_LEV       : OUT    std_logic;                       -- Output Level Select (NIM/TTL)
      G_DIR       : OUT    std_logic;                       -- Output Enable
      G_DOUT      : OUT    std_logic_vector (1 DOWNTO 0);   -- Out G - LEMO (2 x NIM/TTL)
      G_DIN       : IN     std_logic_vector (1 DOWNTO 0);   -- In G - LEMO (2 x NIM/TTL)
      --*************************************************
      -- A395x MEZZANINES INTERFACES (PORT D,E,F)
      --*************************************************
      -- Expansion Mezzanine Identifier:
      -- x_IDCODE :
      -- 000 : A395A (32 x IN LVDS/ECL)
      -- 001 : A395B (32 x OUT LVDS)
      -- 010 : A395C (32 x OUT ECL)
      -- 011 : A395D (8  x IN/OUT NIM/TTL)
      
      -- Expansion Mezzanine Port Signal Standard Select
      -- x_LEV : 
      --    0=>TTL,1=>NIM

      -- Expansion Mezzanine Port Direction
      -- x_DIR : 
      --    0=>OUT,1=>IN

      -- In/Out D (I/O Expansion)
      D_IDCODE    : IN     std_logic_vector ( 2 DOWNTO 0);  -- D slot mezzanine Identifier
      D_LEV       : OUT    std_logic;                       -- D slot Port Signal Level Select 
      D_DIR       : OUT    std_logic;                       -- D slot Port Direction
      D_DIN       : IN     std_logic_vector (31 DOWNTO 0);  -- D slot Data In  Bus
      D_DOUT      : OUT    std_logic_vector (31 DOWNTO 0);  -- D slot Data Out Bus
      -- In/Out E (I/O Expansion)
      E_IDCODE    : IN     std_logic_vector ( 2 DOWNTO 0);  -- E slot mezzanine Identifier
      E_LEV       : OUT    std_logic;                       -- E slot Port Signal Level Select
      E_DIR       : OUT    std_logic;                       -- E slot Port Direction
      E_DIN       : IN     std_logic_vector (31 DOWNTO 0);  -- E slot Data In  Bus
      E_DOUT      : OUT    std_logic_vector (31 DOWNTO 0);  -- E slot Data Out Bus
      -- In/Out F (I/O Expansion)
      F_IDCODE    : IN     std_logic_vector ( 2 DOWNTO 0);  -- F slot mezzanine Identifier
      F_LEV       : OUT    std_logic;                       -- F slot Port Signal Level Select
      F_DIR       : OUT    std_logic;                       -- F slot Port Direction
      F_DIN       : IN     std_logic_vector (31 DOWNTO 0);  -- F slot Data In  Bus
      F_DOUT      : OUT    std_logic_vector (31 DOWNTO 0);  -- F slot Data Out Bus
      --*************************************************
      -- DELAY LINES
      --*************************************************
      -- PDL = Programmable Delay Lines  (Step = 0.25ns / FSR = 64ns)
      -- DLO = Delay Line Oscillator     (Half Period ~ 10 ns)
      -- 3D3428 PDL (PROGRAMMABLE DELAY LINE) CONFIGURATION
      PDL_WR      : OUT    std_logic;                       -- Write Enable
      PDL_SEL     : OUT    std_logic;                       -- PDL Selection (0=>PDL0, 1=>PDL1)
      PDL_READ    : IN     std_logic_vector ( 7 DOWNTO 0);  -- Read Data
      PDL_WRITE   : OUT    std_logic_vector ( 7 DOWNTO 0);  -- Write Data
      PDL_DIR     : OUT    std_logic;                       -- Direction (0=>Write, 1=>Read)
      -- DELAY I/O
      PDL0_OUT    : IN     std_logic;                       -- Signal from PDL0 Output
      PDL1_OUT    : IN     std_logic;                       -- Signal from PDL1 Output
      DLO0_OUT    : IN     std_logic;                       -- Signal from DLO0 Output
      DLO1_OUT    : IN     std_logic;                       -- Signal from DLO1 Output
      PDL0_IN     : OUT    std_logic;                       -- Signal to   PDL0 Input
      PDL1_IN     : OUT    std_logic;                       -- Signal to   PDL1 Input
      DLO0_GATE   : OUT    std_logic;                       -- DLO0 Gate (active high)
      DLO1_GATE   : OUT    std_logic;                       -- DLO1 Gate (active high)
      --*************************************************
      -- SPARE PORTS
      --*************************************************
      SPARE_OUT    : OUT   std_logic_vector(11 downto 0);   -- SPARE Data Out 
      SPARE_IN     : IN    std_logic_vector(11 downto 0);   -- SPARE Data In
      SPARE_DIR    : OUT   std_logic_vector(11 downto 0);   -- SPARE Direction (0 => OUT, 1 => IN)   
      --*************************************************
      -- LED
      --*************************************************
      RED_PULSE       : OUT    std_logic;                   -- RED   Led Pulse (active high)
      GREEN_PULSE     : OUT    std_logic                    -- GREEN Led Pulse (active high)
   );

-- Declarations

END coin_reference ;

ARCHITECTURE rtl OF coin_reference IS

-- Registers
signal A_STATUS   : std_logic_vector(31 downto 0); -- R
signal B_STATUS   : std_logic_vector(31 downto 0); -- R
signal C_STATUS   : std_logic_vector(31 downto 0); -- R
signal A_MASK     : std_logic_vector(31 downto 0); -- W
signal B_MASK     : std_logic_vector(31 downto 0); -- W
signal C_MASK     : std_logic_vector(31 downto 0); -- W
signal GATEWIDTH  : std_logic_vector(15 downto 0); -- W
signal C_CONTROL  : std_logic_vector(31 downto 0); -- W
signal D_CONTROL  : std_logic_vector(31 downto 0); -- R/W
signal E_CONTROL  : std_logic_vector(31 downto 0); -- R/W
signal F_CONTROL  : std_logic_vector(31 downto 0); -- R/W
signal G_CONTROL  : std_logic_vector(31 downto 0); -- W
signal D_DATA     : std_logic_vector(31 downto 0); -- R/W
signal E_DATA     : std_logic_vector(31 downto 0); -- R/W
signal F_DATA     : std_logic_vector(31 downto 0); -- R/W
signal MODE       : std_logic_vector(15 downto 0); -- W
signal SCRATCH    : std_logic_vector(15 downto 0); -- R/W
signal REVISION   : std_logic_vector(15 downto 0); -- R
signal PDL_CONTROL: std_logic_vector(15 downto 0); -- W
signal PDL_DATA   : std_logic_vector(15 downto 0); -- R/W

-- Register Bits
-- MODE Register
signal DELAY_SEL  : std_logic_vector(1 downto 0); -- "00" : PDL0 => Programmable Delay Line 0
                                                  -- "01" : PDL1 => Programmable Delay Line 0
                                                  -- "10" : DLO0  => Gated Delay Line Oscillator 0
                                                  -- "11" : DLO1  => Gated Delay Line Oscillator 1
signal UNIT_MODE  : std_logic; -- '0' : Coincidence Unit; '1' : I/O Register
signal OPERATOR   : std_logic; -- '0' : AND ; '1' : OR
signal PULSE_MODE : std_logic; --

-- Local Signals
signal A     : std_logic_vector(31 downto 0);
signal B     : std_logic_vector(31 downto 0);
signal C     : std_logic_vector(31 downto 0);

-- Coincidences
signal COINC       : std_logic;
signal EXT_GATE    : std_logic;   
signal DLO_GATE    : std_logic;   
signal PDL_GATE    : std_logic;   
signal STARTDELAY  : std_logic;
signal STOPDELAY   : std_logic; 
signal STOP_PDL    : std_logic; 
signal STOP_DLO    : std_logic; 
signal CNT         : std_logic_vector(15 downto 0);
signal DLO_PULSE   : std_logic;
signal PDL_PULSEOUT: std_logic;
signal DLO_PULSEOUT: std_logic;

-- WAVEFORM GENERATOR
signal WVF_CNT     : std_logic_vector(4 downto 0) := (others => '0');
signal PDL_IN_i    : std_logic;
signal ENABLE_CNT  : std_logic;

-- ****************************************************************************
-- Custom local signals and registers
-- ****************************************************************************

-- Custom Registers
signal CREVISION     : std_logic_vector(15 downto 0); --R
signal CSCRATCH      : std_logic_vector(15 downto 0); --R/W
signal CUSTOM_CNTRL  : std_logic_vector(15 downto 0); --W
signal CUSTOM_STATUS : std_logic_vector(15 downto 0); --R
signal COINC_RES_REG : std_logic_vector(15 downto 0); --W
signal SI_GATEWIDTH  : std_logic_vector(15 downto 0); --W
signal SI_RES_DELAY  : std_logic_vector(15 downto 0); --W
signal SI_DELAY      : logic_vector16_array(3 downto 0); --W
signal BGO_DELAY_T_SHORT: std_logic_vector(15 downto 0); --W
signal BGO_DELAY_T_LONG : std_logic_vector(15 downto 0); --W
signal ATRIG_REG     : std_logic_vector(31 downto 0); --R
--signal BTRIG_REG     : std_logic_vector(31 downto 0); --R
signal COINC_ARRAY   : std_logic_vector(15 downto 0); --R
--signal COINC_REG     : std_logic_vector(15 downto 0); --R
signal COINC_MASK    : std_logic_vector(15 downto 0); --R
signal COUNTER_REG   : std_logic_vector(63 downto 0); --R
signal A_TIME        : logic_vector32_array(0 to 31); --R
signal B_TIME        : logic_vector32_array(0 to 31); --R

--Custom Register Bits
signal EXT_COINC_RES : std_logic;
--signal COINC_OUT     : std_logic;
signal FLOOD_MODE    : std_logic;

-- Custom Signals
--signal A_LOW         : std_logic;
--signal A_HIGH        : std_logic;
-- Timers
signal COUNTER       : std_logic_vector(63 downto 0);
signal TIME_AFTER_RST: std_logic_vector(31 downto 0);
signal TIME_AFTER_COINC: std_logic_vector(31 downto 0);

--signal TIME_EN       : std_logic;
--signal TIME_EN       : std_logic_vector(31 downto 0);
signal A_TRIG			: std_logic_vector(31 downto 0);

--Si signals and counters
signal SI_FOR			: std_logic_vector(3 downto 0);
--signal SI_TRIGGER    : std_logic;
--signal START_SITRIG  : std_logic;
--signal STOP_SITRIG   : std_logic;
--signal SI_TRIG_CNT   : std_logic_vector(15 downto 0);
signal SI_RESET		: std_logic_vector(3 downto 0);
signal SI_RESET_CNT	: logic_vector16_array(3 downto 0);
--signal START_SIRES1  : std_logic;
--signal SI_RES1_CNT   : std_logic_vector(15 downto 0);
--signal START_SIRES2  : std_logic;
--signal SI_RES2_CNT   : std_logic_vector(15 downto 0);
--signal START_SIRES3  : std_logic;
--signal SI_RES3_CNT   : std_logic_vector(15 downto 0);
--signal START_SIRES4  : std_logic;
--signal SI_RES4_CNT   : std_logic_vector(15 downto 0);
--signal START_SITDEL1 : std_logic;
--signal STOP_SITDEL1  : std_logic;
--signal SI_TDEL1_CNT  : std_logic_vector(8 downto 0);
--signal START_SITRIG1 : std_logic;
--signal STOP_SITRIG1  : std_logic;
--signal SI_TRIG1_CNT  : std_logic_vector(8 downto 0);
--signal START_SITDEL2 : std_logic;
--signal STOP_SITDEL2  : std_logic;
--signal SI_TDEL2_CNT  : std_logic_vector(8 downto 0);
--signal START_SITRIG2 : std_logic;
--signal STOP_SITRIG2  : std_logic;
--signal SI_TRIG2_CNT  : std_logic_vector(8 downto 0);
--signal START_SITDEL3 : std_logic;
--signal STOP_SITDEL3  : std_logic;
--signal SI_TDEL3_CNT  : std_logic_vector(8 downto 0);
--signal START_SITRIG3 : std_logic;
--signal STOP_SITRIG3  : std_logic;
--signal SI_TRIG3_CNT  : std_logic_vector(8 downto 0);
--signal START_SITDEL4 : std_logic;
--signal STOP_SITDEL4  : std_logic;
--signal SI_TDEL4_CNT  : std_logic_vector(8 downto 0);
--signal START_SITRIG4 : std_logic;
--signal STOP_SITRIG4  : std_logic;
--signal SI_TRIG4_CNT  : std_logic_vector(8 downto 0);
signal BLOCK_SI		: std_logic_vector(3 downto 0);
signal BLOCK_SI_CNT  : logic_vector16_array(3 downto 0);
signal SI_WINDOW		: std_logic_vector(3 downto 0);
--signal SI_STOP_WINDOW: std_logic_vector(3 downto 0);
signal SI_GATE_CNT	: logic_vector16_array(3 downto 0);
--signal SI_WINDOW_DELAY: std_logic_vector(3 downto 0);
--signal SI_TIME_EN		: std_logic_vector(3 downto 0);
signal SI_START		: logic_vector16_array(3 downto 0);
signal SI_STOP			: logic_vector16_array(3 downto 0);
signal SI_TIME			: logic_vector32_array(3 downto 0);

--BGO signals and counters
signal BGO_TRIGGER_SHORT: std_logic_vector(31 downto 0);
signal BGO_TRIGGER_LONG : std_logic_vector(31 downto 0);
signal MIN_BGO_TRIGGER_WIDTH : std_logic_vector(7 downto 0);
signal BGO_WINDOW		: std_logic_vector(3 downto 0);
signal BGO_WINDOW_RAW: std_logic_vector(3 downto 0);
signal BGO_WINDOW_CNT: logic_vector8_array(3 downto 0);
signal BLOCK_BGO		: std_logic_vector(31 downto 0);
signal BLOCK_BGO_CNT : logic_vector16_array(31 downto 0); 
signal BLOCK_BGO_DELAY : std_logic_vector(15 downto 0);
signal BGO_START_SHORT : std_logic_vector(15 downto 0);
signal BGO_START_LONG: std_logic_vector(15 downto 0); 
signal BGO_STOP_SHORT: std_logic_vector(15 downto 0);
signal BGO_STOP_LONG	: std_logic_vector(15 downto 0);

--COINC signals
signal COINC_SIGNAL  : std_logic_vector(15 downto 0); 

BEGIN

   --*************************************************
   -- USER DESIGN REVISION
   --*************************************************
   REVISION  <= X"0101"; -- 1.1
   
   --*************************************************
   -- CUSTOM DESIGN REVISION
   --*************************************************
   CREVISION  <= X"0104"; -- 1.01

   --*************************************************
   -- SPARE PORT
   --*************************************************
   -- ALL Outputs driving 0
   SPARE_OUT <= (others => '0');
   SPARE_DIR <= (others => '1');

   --*************************************************
   -- LED PULSES
   --*************************************************
   RED_PULSE   <= '0';
   GREEN_PULSE <= EXT_GATE;

   --*************************************************
   -- PORT SIGNAL STANDARD SELECTION                             
   --*************************************************
   -- Ports D,E,F,G signal standard set by register.
   D_LEV <= D_CONTROL(0);
   E_LEV <= E_CONTROL(0);
   F_LEV <= F_CONTROL(0);

   --*************************************************
   -- PORT DIRECTION
   --*************************************************
   -- Ports D,E,F,G set by register.
   D_DIR  <= D_CONTROL(1);
   E_DIR  <= E_CONTROL(1);
   F_DIR  <= F_CONTROL(1);

   --*************************************************
   -- PORT G DIRECTION & LEVEL
   --*************************************************   
   -- Port G direction is not user controllable
   -- because we want port G to be used as a GATE output.
   -- G_CONTROL register could be used to control port G
   -- direction.
   G_LEV <= G_CONTROL(0); 
   G_DIR  <= '0';         -- Port G is Output only

   --*************************************************
   -- PORT DATA OUT
   --*************************************************
   -- Ports D,E,F are driven by registers.
   D_DOUT <= D_DATA;
   E_DOUT <= E_DATA;
   F_DOUT <= F_DATA;

   --*************************************************
   -- GATE ON EXTERNAL CONNECTOR   
   --*************************************************
   -- EXT_GATE drives G connector.  
   --  Port G MUST be configured as an output.
   --               __________________________
   -- EXT_GATE  ___|                          |_______
   --               __________________________
   -- G_DOUT(0) ___|                          |_______
   --           ___                            _______
   -- G_DOUT(1)    |__________________________|
   --
   G_DOUT(0)  <= EXT_GATE;
   G_DOUT(1)  <= EXT_GATE;

   --*************************************************
   -- PDL DELAY LINES CONTROL
   --*************************************************
   -- PDLs can be configured by two registers:
   --  * PDL_CONTROL
   --  * PDL_DATA
   PDL_WR     <= PDL_CONTROL(0);
   PDL_DIR    <= PDL_CONTROL(1);
   PDL_SEL    <= PDL_CONTROL(2);
   PDL_WRITE  <= PDL_DATA(7 downto 0);
   --- END OF DELAY LINES CONTROLS
   
   A_STATUS <= A_DIN;
   B_STATUS <= B_DIN;
   C_STATUS <= C;
   
   --*************************************************
   -- MODE Register Bit Mapping       
   --*************************************************
   DELAY_SEL(0) <= MODE(0);
   DELAY_SEL(1) <= MODE(1);
   UNIT_MODE    <= MODE(3);
   OPERATOR     <= MODE(4);
   PULSE_MODE   <= MODE(5);

   --*************************************************
   -- CUSTOM_CNTRL Register Bit Mapping       
   --*************************************************
   FLOOD_MODE <= CUSTOM_CNTRL(0);

	--*********
	--RESET PLL
	--*********
	PROCESS(nLBRES)
	begin
		if nLBRES = '0' then
			PLL_RESET<='1';
		else
			PLL_RESET<='0';
		end if;
	end process;



   --*************************************************
   -- CUSTOM_STATUS Register Bit Mapping       
   --*************************************************
--   CUSTOM_STATUS <= X"000" & "000" & COINC_OUT;
   CUSTOM_STATUS <= X"000" & "000" & COINC;
   --*************************************************
   -- COINC_RES_REG Register Bit Mapping       
   --*************************************************
   EXT_COINC_RES <= or_reduce(COINC_RES_REG);
  
-- **************set varaiables 

 MIN_BGO_TRIGGER_WIDTH <= X"02" ;
 BGO_START_SHORT <= X"00"& BGO_DELAY_T_SHORT(7 downto 0);
 BGO_STOP_SHORT  <= X"00"& BGO_DELAY_T_SHORT(15 downto 8);
 BGO_START_LONG  <= X"00"& BGO_DELAY_T_LONG(7 downto 0);
 BGO_STOP_LONG   <= X"00"& BGO_DELAY_T_LONG(15 downto 8);
 
 --*************************************************
   -- Inport Port (A,B) Masking
   --*************************************************
   -- Single bits of input port A & B can be masked
   -- through a register.

   -- Masking of Port A 
   A <= A_DIN and A_MASK;
   
   -- Masking of Port B 
   B <= B_DIN and B_MASK;

-- For floating lvds lines   
--   -- Masking of Port A 
--   A <= (not A_DIN) and A_MASK;
--   
--   -- Masking of Port B 
--   B <= (not B_DIN) and B_MASK;


   --**********************************************************
   -- Output Port (C) DRIVER
   --**********************************************************
   --  In this reference design, Port C can be driven
   --  in different mode depending on register content:
   --    (1) UNIT_MODE is driven by a bit of MODE REGISTER 
   --        in order to select Port C Driver. 
   --        If UNIT_MODE is set to '0' then 
   --        Port C is directly driven by C_CONTROL register.
   --        If UNIT_MODE is set to '1' then
   --        Port C is driven by the a logic operation
   --        on A and B port. Which logic operation to adopt
   --        is selected through the OPERATOR signal.
   --        OPERATOR signal is driven by bit 4 of MODE REGISTER.
   
--   -- Select Logic Operation
--   P_OPER_SEL : process(OPERATOR, A, B)
--   begin
--     if OPERATOR = '0' then
--        C <= A and B;
--     else
--        C <= A or B;
--     end if;
--   end process;
   
   -- Select Port C driver based on a configuration bit.
--   P_C_DRIVE: process(UNIT_MODE, C, C_MASK, C_CONTROL)
--   begin
--     if UNIT_MODE = '0' then
--        C_DOUT <= C and C_MASK;
--     else
--        C_DOUT <= C_CONTROL;
--     end if;
--   end process;

   --**********************************************************
   -- Coincidence Processing     
   --**********************************************************
   -- COINC signal is a '1' whenever one bit of C is '1'.
--   COINC <= OR_REDUCE(C);

--   STOPDELAY <= STOP_PDL or STOP_DLO;
   
   -- A '0' to '1' transition on COINC signal 
   -- starts the delay timer.
--   P_COINC: process(COINC, STOPDELAY, nLBRES)
--   begin
--     if STOPDELAY='1' or nLBRES='0' then
--       STARTDELAY <= '0';
--     elsif COINC'event and COINC = '1' then
--       STARTDELAY <= '1';
--     end if;
--   end process;


   --**********************************************************
   -- Custom Coincidence Processing     
   --**********************************************************

   C_DOUT <= C and C_MASK;

   --A_LOW  <= or_reduce(A(13 downto 2));
   --A_HIGH <= or_reduce(A(29 downto 18));

-- ***************** start BGO or trigger block ************----
--generates A_LOW 
--generates A_HIGH
--depends on BGO_TRIGGER
	
--	A_LOW <= or_reduce(BGO_TRIGGER(15 downto 0));
BGO_WINDOW_RAW(0) <= or_reduce(BGO_TRIGGER_LONG(15 downto 0));
BGO_WINDOW_RAW(1) <= or_reduce(BGO_TRIGGER_LONG(31 downto 16));
BGO_WINDOW_RAW(2) <= or_reduce(BGO_TRIGGER_SHORT(15 downto 0));
BGO_WINDOW_RAW(3) <= or_reduce(BGO_TRIGGER_SHORT(31 downto 16));

G_BGO_WINDOW: for i in 0 to 3 generate

	P_BGO_WINDOW_CNT : process (nLBRES, EXT_COINC_RES, PLL_CLK)
		begin 
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				BGO_WINDOW_CNT(i) <= (others => '0' );
			elsif rising_edge(PLL_CLK) then
				if BGO_WINDOW(i) = '0' then 
					BGO_WINDOW_CNT(i) <= (others => '0');
				elsif BGO_WINDOW_RAW(i) = '0' then 
--count only if raw goes to 0; wait while and if it is 1
					BGO_WINDOW_CNT(i) <= BGO_WINDOW_CNT(i)+1;
			end if;
		end if;
	end process;
				
	P_BGO_WINDOW : process (nLBRES, EXT_COINC_RES, PLL_CLK)
		begin
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				BGO_WINDOW(i) <= '0';
			elsif rising_edge (PLL_CLK) then
				if BGO_WINDOW_RAW(i) = '1' then
					BGO_WINDOW(i) <= '1';
				elsif BGO_WINDOW_CNT(i) = MIN_BGO_TRIGGER_WIDTH then
					BGO_WINDOW(i) <= '0';
				end if;
			end if;
		end process;
end generate;

--P_A_LOW_CNT: process(nLBRES, PLL_CLK, EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0'  then
--		   A_LOW_CNT <= (others => '0');
--		elsif rising_edge (PLL_CLK) then
--			if (A_LOW = '0') then 
--				A_LOW_CNT <= (others => '0');
--			elsif or_reduce(BGO_TRIGGER(15 downto 0)) = '0' then 
--				A_LOW_CNT <=A_LOW_CNT +1;
--			end if;
--		end if;
--	end process;
--
--P_A_LOW: process(nLBRES, PLL_CLK)
--	begin
--		if nLBRES = '0' or EXT_COINC_RES = '0' then
--			A_LOW <= '0';
--		elsif rising_edge(PLL_CLK) then
--			if or_reduce(BGO_TRIGGER(15 downto 0)) ='1' then
--				A_LOW <='1';
--			elsif A_LOW_CNT = MIN_BGO_TRIGGER_WIDTH then
--				A_LOW <='0';
--			end if;
--		end if;
--	end process;
--	
----A_HIGH <= or_reduce (BGO_TRIGGER(31 downto 16));
--P_A_HIGH_CNT: process(nLBRES, PLL_CLK, EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0'  then
--		   A_HIGH_CNT <= (others => '0');
--		elsif rising_edge (PLL_CLK) then
--			if (A_HIGH = '0') then 
--				A_HIGH_CNT <= (others => '0');
--			elsif or_reduce(BGO_TRIGGER(31 downto 16)) = '0' then 
--				A_HIGH_CNT <=A_HIGH_CNT +1;
--			end if;
--		end if;
--	end process;
--
--P_A_HIGH: process(nLBRES, PLL_CLK)
--	begin
--		if nLBRES = '0' or EXT_COINC_RES = '0' then
--			A_HIGH <= '0';
--		elsif rising_edge(PLL_CLK) then
--			if or_reduce(BGO_TRIGGER(31 downto 16)) ='1' then
--				A_HIGH <='1';
--			elsif A_HIGH_CNT = MIN_BGO_TRIGGER_WIDTH then
--				A_HIGH <= '0';
--			end if;
--		end if;
--	end process;

-- ******************** end BGO trigger block ***************-----
	
--   C(15) <= A_LOW;
--   C(31) <= A_HIGH;

   P_COINC_SEL : process(nLBRES, EXT_COINC_RES, PLL_CLK)
	begin
		if nLBRES = '0' or EXT_COINC_RES = '0' then
			COINC <= '0';
		elsif rising_edge(PLL_CLK) then
--assert the global COINC bit
			COINC <= or_reduce(COINC_ARRAY);

--			if FLOOD_MODE = '0' then
--				COINC <= or_reduce(COINC_ARRAY and COINC_MASK);
--			elsif COINC = '0' then
--				COINC <= BGO_WINDOW(2) or BGO_WINDOW(3);
--			end if;
     end if;
   end process;

   EXT_GATE <= STARTDELAY;
   --SI_TRIGGER <= START_SITRIG;

   -- Using both A and B port
--   COINC_ARRAY(31 downto 0)  <= A and B;
--   COINC_ARRAY(32)           <= A(31) and or_reduce(B(3 downto 0));
--   COINC_ARRAY(33)           <= B(31) and or_reduce(A(3 downto 0));
--   COINC_ARRAY(47 downto 34) <= (others => '0');
--
--   C(0) <= EXT_GATE and (COINC_REG(31) or COINC_REG(33));
--   C(1) <= EXT_GATE and (COINC_REG(31) or COINC_REG(32));
--   C(31 downto 2) <= (others => '0');
--
--   A_TIME(4 to 30) <= (others => X"00000000");
--   B_TIME(4 to 30) <= (others => X"00000000");

 -- reroute silicon signals to contguous address space
	
SI_FOR(0) <= A(0);
SI_FOR(1) <= A(16);
SI_FOR(2) <= A(1);
SI_FOR(3) <= A(17);	
	
	-- Definition of COINC_ARRAY

-- ****************-- set COINC tag block ****************------------
-- uses SI_WINDOW for i=1 to 4
-- uses A_LOW
-- uses A_HIGH
-- generates COINC_ARRAY 

COINC_SIGNAL(0) <= SI_WINDOW(0) and SI_WINDOW(1);
COINC_SIGNAL(1) <= SI_WINDOW(2) and SI_WINDOW(3);
COINC_SIGNAL(2) <= SI_WINDOW(0) and SI_WINDOW(3);
COINC_SIGNAL(3) <= SI_WINDOW(2) and SI_WINDOW(1);

COINC_SIGNAL(4) <= SI_WINDOW(0) and BGO_WINDOW(1);
COINC_SIGNAL(5) <= SI_WINDOW(1) and BGO_WINDOW(0);
COINC_SIGNAL(6) <= SI_WINDOW(2) and BGO_WINDOW(1);
COINC_SIGNAL(7) <= SI_WINDOW(3) and BGO_WINDOW(0);

COINC_SIGNAL(8) <= BGO_WINDOW(2) and BGO_WINDOW(3);

COINC_SIGNAL(11 downto 9) <= (others => '0');

--hex=1000 for bgo flood
COINC_SIGNAL(12) <= BGO_WINDOW(2) or BGO_WINDOW(3);

--hex=2000 for si flood
COINC_SIGNAL(13) <= or_reduce(SI_WINDOW(3 downto 0));

COINC_SIGNAL(15 downto 14) <= (others => '0');
 --Change this to procedures so that COINC_ARRAY latches
 G_COINC_ARRAY: for i in 0 to 15 generate
	
	P_COINC_ARRAY: process (nLBRES, EXT_COINC_RES, PLL_CLK)
	begin
		if nLBRES = '0' or EXT_COINC_RES = '0' then
			COINC_ARRAY(i) <= '0';
		elsif rising_edge(PLL_CLK) then
			if COINC_MASK(i) = '1' and COINC_SIGNAL(i) = '1' then
				COINC_ARRAY(i) <= '1';	
			end if;
		end if;
	end process;

end generate;
 


--P_COINC_ARRAY_0: process (nLBRES,EXT_COINC_RES,PLL_CLK)
----Si0 - Si16
--	begin 
--	if nLBRES = '0' or EXT_COINC_RES = '0' then
--		COINC_ARRAY(0) <= '0';
--	elsif rising_edge(PLL_CLK) then
--		if COINC_MASK(0) = '1' and COINC_SIGNAL(0) = '1' then
--				COINC_ARRAY(0) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_1 : process (nLBRES, EXT_COINC_RES,PLL_CLK)
----Si1 - Si17
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' or COINC_MASK(1) = '0' then 
--		COINC_ARRAY(1) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(2) = '1' and SI_WINDOW(3) = '1' then
--			COINC_ARRAY(1) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_2 : process (nLBRES, EXT_COINC_RES,PLL_CLK)
----Si0 - Si17
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' or COINC_MASK(2) = '0' then 
--		COINC_ARRAY(2) <= '0';
--	
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(0) = '1' and SI_WINDOW(3) = '1' then
--			COINC_ARRAY(2) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_3 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----Si1 - Si16
--	begin	
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(3) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(2) = '1' and SI_WINDOW(1) = '1' then
--			COINC_ARRAY(3) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_4 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----Si0 - BGO_R
--	begin 
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(4) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(0) = '1' and BGO_WINDOW(1) = '1' then
--			COINC_ARRAY(4) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_5 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----Si16 - BGO_L
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(5) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(1) = '1' and BGO_WINDOW(0) = '1' then
--			COINC_ARRAY(5) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_6 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----Si1 - BGO_R
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(6) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(2) = '1' and BGO_WINDOW(1) = '1' then
--			COINC_ARRAY(6) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_7 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----Si17 - BGO_L
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(7) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if SI_WINDOW(3) = '1' and BGO_WINDOW(0) = '1' then
--			COINC_ARRAY(7) <= '1';
--		end if;
--	end if;
--end process;
--
--P_COINC_ARRAY_8 : process (nLBRES, EXT_COINC_RES, PLL_CLK)
----BGO_R - BGO_L
--	begin
--	if nLBRES = '0' or EXT_COINC_RES = '0' then 
--		COINC_ARRAY(8) <= '0';
--	elsif rising_edge (PLL_CLK) then
--		if BGO_WINDOW(2) = '1' and BGO_WINDOW(3) = '1' then
--			COINC_ARRAY(8) <= '1';
--		end if;
--	end if;
--end process;

 --COINC_ARRAY(0)           <= SI_WINDOW(0) and SI_WINDOW(1); -- Si-Si
 --COINC_ARRAY(1)           <= SI_WINDOW(2) and SI_WINDOW(3); -- Si-Si
 --  COINC_ARRAY(2)           <= SI_WINDOW(0) and SI_WINDOW(3); -- Si-Si
 --  COINC_ARRAY(3)           <= SI_WINDOW(2) and SI_WINDOW(1); -- Si-Si
 --  COINC_ARRAY(4)           <= SI_WINDOW(0) and A(31);         -- Si-BGO
 --  COINC_ARRAY(5)           <= SI_WINDOW(1) and A(15);         -- Si-BGO
 --  COINC_ARRAY(6)           <= SI_WINDOW(2) and A(31);         -- Si-BGO
 --  COINC_ARRAY(7)           <= SI_WINDOW(3) and A(15);         -- Si-BGO
 --  COINC_ARRAY(8)           <= A(15) and A(31);                 -- BGO-BGO
   

-- **************** --end COINC tag block *************--
	
--	COINC_ARRAY(15 downto 9) <= (others => '0');

   -- Using just A port (Old COINC_ARRAY)
--   COINC_ARRAY(15 downto 0)  <= A(15 downto 0) and A(31 downto 16);
--   COINC_ARRAY(16)           <= A(0)  and A(17);
--   COINC_ARRAY(17)           <= A(0)  and A(18);
--   COINC_ARRAY(18)           <= A(0)  and A(19);
--   COINC_ARRAY(19)           <= A(0)  and A(20);
--   COINC_ARRAY(20)           <= A(0)  and A(21);
--   COINC_ARRAY(21)           <= A(0)  and A(22);
--   COINC_ARRAY(22)           <= A(0)  and A(23);
--   COINC_ARRAY(23)           <= A(0)  and A(24);
--   COINC_ARRAY(24)           <= A(0)  and A(25);
--   COINC_ARRAY(25)           <= A(0)  and A(26);
--   COINC_ARRAY(26)           <= A(0)  and A(27);
--   COINC_ARRAY(27)           <= A(0)  and A(28);
--   COINC_ARRAY(28)           <= A(0)  and A(29);
--   COINC_ARRAY(29)           <= A(0)  and A(30);
--   COINC_ARRAY(30)           <= A(0)  and A(31);
--   COINC_ARRAY(31)           <= A(16) and A(1);
--   COINC_ARRAY(32)           <= A(16) and A(2);
--   COINC_ARRAY(33)           <= A(16) and A(3);
--   COINC_ARRAY(34)           <= A(16) and A(4);
--   COINC_ARRAY(35)           <= A(16) and A(5);
--   COINC_ARRAY(36)           <= A(16) and A(6);
--   COINC_ARRAY(37)           <= A(16) and A(7);
--   COINC_ARRAY(38)           <= A(16) and A(8);
--   COINC_ARRAY(39)           <= A(16) and A(9);
--   COINC_ARRAY(40)           <= A(16) and A(10);
--   COINC_ARRAY(41)           <= A(16) and A(11);
--   COINC_ARRAY(42)           <= A(16) and A(12);
--   COINC_ARRAY(43)           <= A(16) and A(13);
--   COINC_ARRAY(44)           <= A(16) and A(14);
--   COINC_ARRAY(45)           <= A(16) and A(15);
--   COINC_ARRAY(46)           <= A(1)  and A(18);
--   COINC_ARRAY(47)           <= A(1)  and A(19);
--   COINC_ARRAY(48)           <= A(1)  and A(20);
--   COINC_ARRAY(49)           <= A(1)  and A(21);
--   COINC_ARRAY(50)           <= A(1)  and A(22);
--   COINC_ARRAY(51)           <= A(1)  and A(23);
--   COINC_ARRAY(52)           <= A(1)  and A(24);
--   COINC_ARRAY(53)           <= A(1)  and A(25);
--   COINC_ARRAY(54)           <= A(1)  and A(26);
--   COINC_ARRAY(55)           <= A(1)  and A(27);
--   COINC_ARRAY(56)           <= A(1)  and A(28);
--   COINC_ARRAY(57)           <= A(1)  and A(29);
--   COINC_ARRAY(58)           <= A(1)  and A(30);
--   COINC_ARRAY(59)           <= A(1)  and A(31);
--   COINC_ARRAY(60)           <= A(17) and A(2);
--   COINC_ARRAY(61)           <= A(17) and A(3);
--   COINC_ARRAY(62)           <= A(17) and A(4);
--   COINC_ARRAY(63)           <= A(17) and A(5);
--   COINC_ARRAY(64)           <= A(17) and A(6);
--   COINC_ARRAY(65)           <= A(17) and A(7 );
--   COINC_ARRAY(66)           <= A(17) and A(8);
--   COINC_ARRAY(67)           <= A(17) and A(9);
--   COINC_ARRAY(68)           <= A(17) and A(10);
--   COINC_ARRAY(69)           <= A(17) and A(11);
--   COINC_ARRAY(70)           <= A(17) and A(12);
--   COINC_ARRAY(71)           <= A(17) and A(13);
--   COINC_ARRAY(72)           <= A(17) and A(14);
--   COINC_ARRAY(73)           <= A(17) and A(15);
--   COINC_ARRAY(95 downto 74) <= (others => '0');

-- ****************** begin SILICON TRIGGER generate block ***************--
--depends on COINC
--depends on SI_WINDOW(3 downto 0)
--uses SI_GATEWIDTH

G_SI_GATE: for i in 0 to 3 generate
	
	P_SI_GATE : process (PLL_CLK,nLBRES,EXT_COINC_RES)
		begin
			if (nLBRES='0' or EXT_COINC_RES='0') then
				C(i) <= '0';
			elsif rising_edge(PLL_CLK) then
				if (SI_WINDOW(i)= '1' and COINC= '1') then
					C(i) <= '1';
				elsif SI_GATE_CNT(i) = SI_GATEWIDTH then
					C(i) <= '0';
				end if;
			end if;
		end process;
	
	P_SI_GATE_CNT : process (PLL_CLK, nLBRES, EXT_COINC_RES)
		begin
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				SI_GATE_CNT(i) <= (others => '0');
			elsif rising_edge (PLL_CLK) then 
				if C(i) = '0' then
					SI_GATE_CNT(i) <= (others => '0');
				else
					SI_GATE_CNT(i) <= SI_GATE_CNT(i) +1;
				end if;
			end if;
		end process;
					
end generate;

--   C(0)            <= SI_TRIGGER and ((COINC_REG(0) or COINC_REG(2)) or COINC_REG(4));
--   C(1)            <= SI_TRIGGER and ((COINC_REG(0) or COINC_REG(3)) or COINC_REG(5));
--   C(2)            <= SI_TRIGGER and ((COINC_REG(1) or COINC_REG(3)) or COINC_REG(6));
--   C(3)            <= SI_TRIGGER and ((COINC_REG(1) or COINC_REG(2)) or COINC_REG(7));



	C(7 downto 4)	 <= SI_WINDOW(3 downto 0);
	C(8)  			 <= COINC;
--	C(13 downto 9)  <= (others => '0');
	C(9)				<= BLOCK_SI(0);
	C(13 downto 10) <= BLOCK_BGO(5 downto 2);
	C(14) <= BGO_WINDOW(0);
	C(15) <= BGO_WINDOW(2);
	C(19 downto 16) <= SI_RESET(3 downto 0);
--C(29 downto 20) <= (others => '0');
	C(20) <= BLOCK_SI(1);
   C(29 downto 21) <= BLOCK_BGO(26 downto 18);
	C(30) <= BGO_WINDOW(1);
	C(31) <= BGO_WINDOW(3);

--   A_TIME(5  to 15) <= (others => X"00000000");
--   A_TIME(21 to 31) <= (others => X"00000000");
   B_TIME(0  to 31) <= (others => X"00000000");



	
	
	-- Begin silicon trigger delay processes
-- depends on A(0 upto 3)
-- depends on C(0 upto 3)
-- depends on START_SIDELi for i=1 to 4
-- depends on COINC 
-- uses SI_DELAY_i for i=1 to 4 
-- generates START_SITRIGi for i=1 to 4 (used in setting COINC_ARRAY)
-- generates STOP_SITRIGi for i=1 to 4 (used to start START_SIDELi)



G_SI_BLOCK: for i in 0 to 3 generate
	
	SI_START(i) <= X"00" & SI_DELAY(i)(7 downto 0);
	SI_STOP(i)  <= X"00" & SI_DELAY(i)(15 downto 8);


	P_BLOCK_SI: process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BLOCK_SI(i) <= '0';
				elsif rising_edge (PLL_CLK) then
					if COINC = '0' and SI_FOR(i) = '1' then
						BLOCK_SI(i) <= '1';
--					elsif BLOCK_SI_CNT(i) = SI_RES_DELAY then
					elsif C(i) = '1' then
						BLOCK_SI(i) <= '0';
					elsif SI_RESET(i) = '1' then
						BLOCK_SI(i) <= '0';
					end if;
				end if;
			end process;
			
		P_BLOCK_SI_CNT: process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BLOCK_SI_CNT(i) <= (others => '0');
				elsif rising_edge (PLL_CLK) then
					if BLOCK_SI(i) = '0' then
						BLOCK_SI_CNT(i) <= (others => '0');
					else
						BLOCK_SI_CNT (i) <= BLOCK_SI_CNT(i) + 1;
					end if;
				end if;
			end process;
		
		P_SI_WINDOW : process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					SI_WINDOW(i) <= '0';
				elsif rising_edge(PLL_CLK) then
					if BLOCK_SI_CNT (i) = SI_START(i) then
						SI_WINDOW(i) <= '1' ;
					elsif BLOCK_SI_CNT (i)= SI_STOP(i) then
						SI_WINDOW(i) <= '0' ;
					elsif BLOCK_SI(i) = '0' then
						SI_WINDOW(i) <= '0';
					end if;
				end if;
			end process;

		P_SI_RESET: process(nLBRES, EXT_COINC_RES, PLL_CLK)
		begin
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				SI_RESET(i) <= '0';
			elsif rising_edge(PLL_CLK) then
				if TIME_AFTER_RST = X"00000002" then
-- short time after reset issue a reset for silicon modules
					SI_RESET(i) <= '1';
				elsif BLOCK_SI_CNT(i) = SI_RES_DELAY then
					SI_RESET(i) <= '1';
				elsif SI_RESET_CNT(i) = SI_GATEWIDTH then
					SI_RESET(i) <= '0';
				end if;
			end if;
		end process;
	
		P_SI_RESET_CNT: process(nLBRES, EXT_COINC_RES, PLL_CLK)
		begin
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				SI_RESET_CNT(i) <= (others => '0');
			elsif rising_edge(PLL_CLK) then
				if SI_RESET(i) = '0' then
					SI_RESET_CNT(i) <= (others => '0');
				else
					SI_RESET_CNT(i) <= SI_RESET_CNT(i) + 1;
				end if;
			end if;
		end process;
	end generate;
--
--   P_START_SITDEL1: process(nLBRES, EXT_COINC_RES, PLL_CLK, STOP_SITDEL1)
--   begin
--     if STOP_SITDEL1 = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW_DELAY(0) <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if COINC= '0' and A(0) = '1' then
--         SI_WINDOW_DELAY(0) <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITDEL1: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(0))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(0) = '0' then
--       STOP_SITDEL1    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TDEL1_CNT = SI_DELAY_0(7 downto 0) then
--         STOP_SITDEL1  <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TDEL1_CNT: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(0))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(0) = '0' then
--       SI_TDEL1_CNT <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TDEL1_CNT <= SI_TDEL1_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITRIG1: process(nLBRES, EXT_COINC_RES, PLL_CLK, SI_STOP_WINDOW(0))
--   begin
--     if SI_STOP_WINDOW(0) = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--		SI_WINDOW(0)   <= '0';
--     elsif rising_edge(PLL_CLK) then
--		 if STOP_SITDEL1 = '1' then
--         SI_WINDOW(0) <= '1';
--       elsif C(0)= '1' then
--			SI_WINDOW(0) <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITRIG1: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_STOP_WINDOW(0)  <= '0';
--     elsif rising_edge(PLL_CLK) then
--	      if SI_TRIG1_CNT = SI_DELAY_0(15 downto 8) then
--				SI_STOP_WINDOW(0)  <= '1';
--			elsif SI_RESET(0) <= '1' then
--				SI_STOP_WINDOW(0) <='0';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TRIG1_CNT: process(nLBRES, PLL_CLK, SI_WINDOW)
--   begin
--     if nLBRES = '0' or SI_WINDOW(0) = '0' then
--       SI_TRIG1_CNT  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TRIG1_CNT  <= SI_TRIG1_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITDEL2: process(nLBRES, EXT_COINC_RES, PLL_CLK, STOP_SITDEL2)
--   begin
--     if STOP_SITDEL2 = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW_DELAY(1) <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if COINC = '0' and A(16) = '1' then
--         SI_WINDOW_DELAY(1) <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITDEL2: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(1))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(1) = '0' then
--       STOP_SITDEL2    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TDEL2_CNT = SI_DELAY_1(7 downto 0) then
--         STOP_SITDEL2  <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TDEL2_CNT: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(1))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(1) = '0' then
--       SI_TDEL2_CNT <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TDEL2_CNT <= SI_TDEL2_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITRIG2: process(nLBRES, EXT_COINC_RES, PLL_CLK, SI_STOP_WINDOW(1))
--   begin
--     if SI_STOP_WINDOW(1) = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW(1)  <= '0';
--     elsif rising_edge(PLL_CLK) then
--		 if STOP_SITDEL2 = '1' then
--         SI_WINDOW(1) <= '1';
--       elsif C(1) = '1' then
--			SI_WINDOW(1) <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITRIG2: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_STOP_WINDOW(1)  <= '0';
--     elsif rising_edge(PLL_CLK) then
-- 		if SI_TRIG2_CNT = SI_DELAY_1(15 downto 8) then
--         SI_STOP_WINDOW(1)  <= '1';
--		elsif SI_RESET(1) = '1' then 
--			SI_STOP_WINDOW(1)  <= '0';
--		end if;
--     end if;
--   end process;
--
--   P_SI_TRIG2_CNT: process(nLBRES, PLL_CLK, SI_WINDOW(1))
--   begin
--     if nLBRES = '0' or SI_WINDOW(1) = '0' then
--       SI_TRIG2_CNT  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TRIG2_CNT  <= SI_TRIG2_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITDEL3: process(nLBRES, EXT_COINC_RES, PLL_CLK, STOP_SITDEL3)
--   begin
--     if STOP_SITDEL3 = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW_DELAY(2) <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if COINC = '0' and A(1) = '1' then
--         SI_WINDOW_DELAY(2) <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITDEL3: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(2))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(2) = '0' then
--       STOP_SITDEL3    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TDEL3_CNT = SI_DELAY_2(7 downto 0) then
--         STOP_SITDEL3  <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TDEL3_CNT: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(2))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(2) = '0' then
--       SI_TDEL3_CNT <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TDEL3_CNT <= SI_TDEL3_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITRIG3: process(nLBRES, EXT_COINC_RES, PLL_CLK, SI_STOP_WINDOW(2))
--   begin
--     if SI_STOP_WINDOW(2) = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW(2)   <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if STOP_SITDEL3 = '1' then
--         SI_WINDOW(2) <= '1';
--		 elsif C(2) = '1' then
--			SI_WINDOW(2) <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITRIG3: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_STOP_WINDOW(2)    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TRIG3_CNT = SI_DELAY_2(15 downto 8) then
--         SI_STOP_WINDOW(2)  <= '1';
--		  elsif SI_RESET(2) = '1' then
--			SI_STOP_WINDOW(2)  <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TRIG3_CNT: process(nLBRES, PLL_CLK, SI_WINDOW(2))
--   begin
--     if nLBRES = '0' or SI_WINDOW(2) = '0' then
--       SI_TRIG3_CNT  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TRIG3_CNT  <= SI_TRIG3_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITDEL4: process(nLBRES, EXT_COINC_RES, PLL_CLK, STOP_SITDEL4)
--   begin
--     if STOP_SITDEL4 = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW_DELAY(3) <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if COINC = '0' and A(17) = '1' then
--         SI_WINDOW_DELAY(3) <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITDEL4: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(3))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(3) = '0' then
--       STOP_SITDEL4    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TDEL4_CNT = SI_DELAY_3(7 downto 0) then
--         STOP_SITDEL4  <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TDEL4_CNT: process(nLBRES, PLL_CLK, SI_WINDOW_DELAY(3))
--   begin
--     if nLBRES = '0' or SI_WINDOW_DELAY(3) = '0' then
--       SI_TDEL4_CNT <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TDEL4_CNT <= SI_TDEL4_CNT + 1;
--     end if;
--   end process;
--
--   P_START_SITRIG4: process(nLBRES, EXT_COINC_RES, PLL_CLK, SI_STOP_WINDOW(3))
--   begin
--     if SI_STOP_WINDOW(3) = '1' or (nLBRES = '0' or EXT_COINC_RES = '0') then
--       SI_WINDOW(3)   <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if STOP_SITDEL4 = '1' then
--         SI_WINDOW(3) <= '1';
--		 elsif C(3) = '1' then
--			SI_WINDOW(3) <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_STOP_SITRIG4: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_STOP_WINDOW(3)    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TRIG4_CNT = SI_DELAY_3(15 downto 8) then
--         SI_STOP_WINDOW(3)  <= '1';
--		  elsif SI_RESET(3) = '1' then
--			SI_STOP_WINDOW(3)  <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TRIG4_CNT: process(nLBRES, PLL_CLK, SI_WINDOW(3))
--   begin
--     if nLBRES = '0' or SI_WINDOW(3) = '0' then
--       SI_TRIG4_CNT  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TRIG4_CNT  <= SI_TRIG4_CNT + 1;
--     end if;
--   end process;

-- End silicon trigger delay processes

-- Begin BGO trigger delay processes
BLOCK_BGO(1 downto 0) <= (others => '0');
BLOCK_BGO(17 downto 16) <= (others => '0');
BGO_TRIGGER_LONG(1 downto 0) <= (others => '0');
BGO_TRIGGER_LONG(17 downto 16) <= (others => '0');
BGO_TRIGGER_SHORT(1 downto 0) <= (others => '0');
BGO_TRIGGER_SHORT(17 downto 16) <= (others => '0');

GEN_BGO: for i in 0 to 31 generate
	GEN_BGO_COND: if i mod 16 > 1 generate
		P_BLOCK_BGO: process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BLOCK_BGO(i) <= '0';
				elsif rising_edge (PLL_CLK) then
					if BLOCK_BGO_CNT(i) = BLOCK_BGO_DELAY then
						BLOCK_BGO(i) <= '0';
					elsif COINC = '0' and A(i) = '1' then
						BLOCK_BGO(i) <= '1';
					end if;
				end if;
			end process;
			
		P_BLOCK_BGO_CNT: process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BLOCK_BGO_CNT(i) <= (others => '0');
				elsif rising_edge (PLL_CLK) then
					if BLOCK_BGO(i) = '0' then
						BLOCK_BGO_CNT(i) <= (others => '0');
					else
						BLOCK_BGO_CNT (i) <= BLOCK_BGO_CNT(i) + 1;
					end if;
				end if;
			end process;
		
		P_BGO_TRIGGER_SHORT : process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BGO_TRIGGER_SHORT(i) <= '0';
				elsif rising_edge(PLL_CLK) then
					if BLOCK_BGO_CNT (i) = BGO_START_SHORT then
						BGO_TRIGGER_SHORT(i) <= '1' ;
					elsif BLOCK_BGO_CNT (i) = BGO_STOP_SHORT then
						BGO_TRIGGER_SHORT(i) <= '0' ;
					end if;
				end if;
			end process;
		
		P_BGO_TRIGGER_LONG : process(nLBRES, EXT_COINC_RES, PLL_CLK)
			begin
				if nLBRES = '0' or EXT_COINC_RES = '0' then
					BGO_TRIGGER_LONG(i) <= '0';
				elsif rising_edge(PLL_CLK) then
					if BLOCK_BGO_CNT (i) = BGO_START_LONG then
						BGO_TRIGGER_LONG(i) <= '1' ;
					elsif BLOCK_BGO_CNT (i) = BGO_STOP_LONG then
						BGO_TRIGGER_LONG(i) <= '0' ;
					end if;
				end if;
			end process;
						
	end generate;
end generate;
-- End BGO trigger delay processes

-- Begin gate generation processes
-- depends on COINC
-- uses GATEWIDTH
-- generates STARTDELAY (used as EXT_GATE)

   P_STARTDELAY: process(nLBRES, EXT_COINC_RES, PLL_CLK)
   begin
     --if STOPDELAY='1' or nLBRES='0' then
		if nLBRES = '0' or EXT_COINC_RES = '0' then
			STARTDELAY <= '0';
		elsif rising_edge(PLL_CLK) then
			if TIME_AFTER_COINC = GATEWIDTH then
				STARTDELAY <= '0';
			elsif TIME_AFTER_COINC = X"0001" then
				STARTDELAY <= '1';
			end if;
		end if;
   end process;

--   P_STOPDELAY: process(nLBRES, PLL_CLK, STARTDELAY)
--   begin
--     if ((nLBRES = '0') or (STARTDELAY = '0')) then
--       STOPDELAY    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if CNT = GATEWIDTH then
--         STOPDELAY  <= '1';
--       end if;
--     end if;
--   end process;

--   P_CNT: process(nLBRES, EXT_COINC_RES, PLL_CLK)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--			CNT <= (others => '0');
--		elsif rising_edge(PLL_CLK) then
--			if STARTDELAY = '0' then
--				CNT <= (others => '0');
--			else
--				CNT <= CNT + 1;
--			end if;
--		end if;
--	end process;
--	  
--	  ((nLBRES = '0') or (STARTDELAY = '0')) then
--       CNT          <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       CNT          <= CNT + 1;
--     end if;
--   end process;
-- End gate generation processes

-- Begin silicon trigger generation processes
-- depends on COINC
-- uses GATEWIDTH
-- generates START_SITRIG (used as SILICON_TRIGGER for outputs C(1 upto 4)) 
	
--	P_START_SITRIG: process(COINC_OUT, STOP_SITRIG, nLBRES)
--   begin
--     if STOP_SITRIG='1' or nLBRES='0' then
--       START_SITRIG <= '0';
--     elsif rising_edge(COINC_OUT) then
--       START_SITRIG <= '1';
--     end if;
--   end process;
--
--   P_STOP_SITRIG: process(nLBRES, PLL_CLK, START_SITRIG)
--   begin
--     if nLBRES = '0' or START_SITRIG = '0' then
--       STOP_SITRIG    <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_TRIG_CNT = SI_GATEWIDTH then
--         STOP_SITRIG  <= '1';
--       end if;
--     end if;
--   end process;
--
--   P_SI_TRIG_CNT: process(nLBRES, PLL_CLK, START_SITRIG)
--   begin
--     if nLBRES = '0' or START_SITRIG = '0' then
--       SI_TRIG_CNT  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--       SI_TRIG_CNT  <= SI_TRIG_CNT + 1;
--     end if;
--   end process;
---- End silicon trigger generation processes

-- Begin silicon reset generation processes
-- depends on STOP_SITRIGi for i=1 to 4
-- uses SI_GATEWIDTH
-- generates START_SIRESi for i=1 to 4 (used on outputs C(16 upto 19))  
	
--GEN_RES: for i in 0 to 3 generate
--	
--	P_START_SIRES: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_RESET(i)   <= '0';
--     elsif rising_edge(PLL_CLK) then
--       if SI_STOP_WINDOW(i) = '1' then        
--				SI_RESET(i) <= '1';
--        elsif SI_RESET_CNT(i) = SI_GATEWIDTH then
--				SI_RESET(i) <= '0';
--       end if;
--     end if;
--   end process;
--
--   P_SI_RESET_CNT: process(nLBRES, PLL_CLK)
--   begin
--     if nLBRES = '0' then
--       SI_RESET_CNT(i)  <= (others => '0');
--     elsif rising_edge(PLL_CLK) then
--		 if SI_RESET(i) = '0' then
--			SI_RESET_CNT(i) <= (others => '0');
--		 else	
--			SI_RESET_CNT(i)  <= SI_RESET_CNT(i) + 1;
--		 end if;
--     end if;
--   end process;
--end generate;
-- End silicon reset generation processes

-- Begin coincidence processes
P_COUNTER: process(nLBRES, PLL_CLK)
   begin
     if nLBRES = '0' then
       COUNTER <= (others => '0');
     elsif rising_edge(PLL_CLK) then
       COUNTER <= COUNTER + 1;
     end if;
   end process;

P_TIME_AFTER_REST: process (nLBRES, EXT_COINC_RES, PLL_CLK) 
	begin
		if nLBRES = '0' or EXT_COINC_RES = '0' then
			TIME_AFTER_RST <= (others=> '0');
		elsif rising_edge(PLL_CLK) then
			TIME_AFTER_RST <= TIME_AFTER_RST + 1;
		end if;
	end process;

P_TIME_AFTER_COINC: process (nLBRES, EXT_COINC_RES, PLL_CLK)
		begin
			if nLBRES = '0' or EXT_COINC_RES = '0' then
				TIME_AFTER_COINC <= (others => '0');
			elsif rising_edge(PLL_CLK) then
				if COINC = '1' then
					TIME_AFTER_COINC <= TIME_AFTER_COINC + 1;
				end if;
			end if;
		end process;

A_TRIG(0) <= BLOCK_SI(0);
A_TRIG(1) <= BLOCK_SI(2);
A_TRIG(15 downto 2) <= BLOCK_BGO(15 downto 2);
A_TRIG(16) <= BLOCK_SI(1);
A_TRIG(17) <= BLOCK_SI(3);
A_TRIG(31 downto 18) <= BLOCK_BGO(31 downto 18);

P_A_TRIG_REG: process (nLBRES, EXT_COINC_RES, PLL_CLK)
	begin 
		if nLBRES = '0' or EXT_COINC_RES = '0' then
			ATRIG_REG <= ( others => '0' );
		elsif rising_edge (PLL_CLK) then
			if TIME_AFTER_COINC = X"0001" then
				ATRIG_REG <= A_TRIG;
			end if;
		end if;
	end process;
	
 --  A_TRIG_REG <= TIME_EN;
	
--   P_BTRIG_REG: process(nLBRES, COINC, EXT_COINC_RES)
--   begin
--    if nLBRES = '0' or EXT_COINC_RES = '0' then
--       BTRIG_REG <= (others => '0');
--     elsif rising_edge(COINC) then
--       if COINC_OUT = '0' then
--         BTRIG_REG <= B;
--       end if;
--     end if;
--   end process;

--   P_COINC_REG: process(nLBRES, COINC, EXT_COINC_RES)
--   begin
--    if nLBRES = '0' or EXT_COINC_RES = '0' then
--       COINC_REG <= (others => '0');
--     elsif rising_edge(COINC) then
--       if COINC_OUT = '0' then
--         COINC_REG <= COINC_ARRAY;
--       end if;
--     end if;
--   end process;
--***********************begin COUNTER block **************--
-- depends on COINC
-- generates COUNTER_REG
   P_COUNTER_REG: process(nLBRES, PLL_CLK, EXT_COINC_RES)
   begin
    if nLBRES = '0' or EXT_COINC_RES = '0' then
       COUNTER_REG <= (others => '0');
 --    elsif rising_edge(COINC) then
	  elsif rising_edge(PLL_CLK) then
			if TIME_AFTER_COINC = X"0001" then
				COUNTER_REG <= COUNTER;
			end if;
     end if;
   end process;
	
--*********************************************************--
-- End coincidence processes

-- Begin time stamp processes
--depend on SI_RESET(i) for si
--depend on BLOCK_BGO(i) for bgo
--depend on SI_WINDOW_DELAY(i) for si

-- bgo time enables
--G_SI_TIME: for i in 0 to 3 generate  
--	P_SI_TIME_EN: process(nLBRES, EXT_COINC_RES, PLL_CLK)
--		begin
--			if nLBRES = '0' or EXT_COINC_RES = '0' then
--				SI_TIME_EN(i) <= '1';
--			elsif rising_edge (PLL_CLK) then
--				if COINC = '1' then
--					SI_TIME_EN(i) <= '0';
--				elsif SI_TIME_EN(i) = '1' and BLOCK_SI(i) = '1' then 
--					SI_TIME_EN(i) <= '0';
--				elsif SI_TIME_EN(i) = '0' and BLOCK_SI(i) = '0' then
--					SI_TIME_EN(i) <= '1';
--				end if;
--			end if;
--		end process;
--end generate;

-- bgo time enables
--G_BGO_TIME: for i in 0 to 31 generate	
--	G_BGO_TIME_FLAG: if i mod 16 >1 generate
--	P_BGO_TIME_EN: process (nLBRES, EXT_COINC_RES, PLL_CLK)
--		begin
--			if nLBRES = '0' or EXT_COINC_RES = '0' then
--				TIME_EN(i) <= '1';
--			elsif rising_edge (PLL_CLK) then
--				if COINC = '1' then
--					TIME_EN(i) <= '0';
--				elsif TIME_EN(i) = '1' and BLOCK_BGO(i) = '1' then
--					TIME_EN(i) <= '0';
--				elsif TIME_EN(i) = '0' and BLOCK_BGO(i) = '0' then
--					TIME_EN(i) <= '1';
--				end if;
--			end if;
--		end process;
--	end generate;
--
--end generate;
--   
--
--TIME_EN(0)   <=  SI_TIME_EN(0);
--TIME_EN(16) <=  SI_TIME_EN(1);
--TIME_EN(1)   <=  SI_TIME_EN(2);
--TIME_EN(17) <=  SI_TIME_EN(3);
	
--P_TIME_EN: process(nLBRES, PLL_CLK, EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       TIME_EN <= '1';
--     elsif rising_edge(PLL_CLK) then
--       if COINC_OUT = '1' then
--         TIME_EN <= '0';
--       end if;
--     end if;
--   end process;

--BGO time
G_TIME_A: for i in 0 to 31 generate
	G_BGO_TIME_FLAG: if i mod 16 >1 generate
		P_TIME_A: process(nLBRES,EXT_COINC_RES,PLL_CLK)
			begin
				if nLBRES= '0' or EXT_COINC_RES = '0' then
					A_TIME(i) <= (others => '0');
				elsif rising_edge(PLL_CLK) then
					if BLOCK_BGO_CNT(i)=X"0001" then
	--if TIME_EN(i) = '1' then
						A_TIME(i) <= COUNTER (31 downto 0);
					end if;
				end if;
			end process;
	end generate;
end generate;

--SI time
G_TIME_SI: for i in 0 to 3 generate
	P_TIME_SI: process(nLBRES,EXT_COINC_RES,PLL_CLK)
		begin
			if nLBRES= '0' or EXT_COINC_RES = '0' then
				SI_TIME(i) <= (others => '0');
			elsif rising_edge(PLL_CLK) then
				if BLOCK_SI_CNT(i)=X"0001" then
--if TIME_EN(i) = '1' then
					SI_TIME(i) <= COUNTER (31 downto 0);
				end if;
			end if;
		end process;
end generate;

A_TIME(0)  <= SI_TIME(0);
A_TIME(1)  <= SI_TIME(2);
A_TIME(16) <= SI_TIME(1);
A_TIME(17) <= SI_TIME(3);


--   P_A0_TIME: process(nLBRES, A(0), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(0) <= (others => '0');
--     elsif rising_edge(A(0)) then
--       if TIME_EN = '1' then
--         A_TIME(0) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A1_TIME: process(nLBRES, A(1), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(1) <= (others => '0');
--     elsif rising_edge(A(1)) then
--       if TIME_EN = '1' then
--         A_TIME(1) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A2_TIME: process(nLBRES, A(2), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(2) <= (others => '0');
--     elsif rising_edge(A(2)) then
--       if TIME_EN = '1' then
--         A_TIME(2) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A3_TIME: process(nLBRES, A(3), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(3) <= (others => '0');
--     elsif rising_edge(A(3)) then
--       if TIME_EN = '1' then
--         A_TIME(3) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A4_TIME: process(nLBRES, A(4), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(4) <= (others => '0');
--     elsif rising_edge(A(4)) then
--       if TIME_EN = '1' then
--         A_TIME(4) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A5_TIME: process(nLBRES, A(5), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(5) <= (others => '0');
--     elsif rising_edge(A(5)) then
--       if TIME_EN = '1' then
--         A_TIME(5) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A6_TIME: process(nLBRES, A(6), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(6) <= (others => '0');
--     elsif rising_edge(A(6)) then
--       if TIME_EN = '1' then
--         A_TIME(6) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A7_TIME: process(nLBRES, A(7), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(7) <= (others => '0');
--     elsif rising_edge(A(7)) then
--       if TIME_EN = '1' then
--         A_TIME(7) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A8_TIME: process(nLBRES, A(8), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(8) <= (others => '0');
--     elsif rising_edge(A(8)) then
--       if TIME_EN = '1' then
--         A_TIME(8) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A9_TIME: process(nLBRES, A(9), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(9) <= (others => '0');
--     elsif rising_edge(A(9)) then
--       if TIME_EN = '1' then
--         A_TIME(9) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A10_TIME: process(nLBRES, A(10), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(10) <= (others => '0');
--     elsif rising_edge(A(10)) then
--       if TIME_EN = '1' then
--         A_TIME(10) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A11_TIME: process(nLBRES, A(11), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(11) <= (others => '0');
--     elsif rising_edge(A(11)) then
--       if TIME_EN = '1' then
--         A_TIME(11) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A12_TIME: process(nLBRES, A(12), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(12) <= (others => '0');
--     elsif rising_edge(A(12)) then
--       if TIME_EN = '1' then
--         A_TIME(12) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A13_TIME: process(nLBRES, A(13), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(13) <= (others => '0');
--     elsif rising_edge(A(13)) then
--       if TIME_EN = '1' then
--         A_TIME(13) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A14_TIME: process(nLBRES, A(14), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(14) <= (others => '0');
--     elsif rising_edge(A(14)) then
--       if TIME_EN = '1' then
--         A_TIME(14) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A15_TIME: process(nLBRES, A(15), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(15) <= (others => '0');
--     elsif rising_edge(A(15)) then
--       if TIME_EN = '1' then
--         A_TIME(15) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A16_TIME: process(nLBRES, A(16), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(16) <= (others => '0');
--     elsif rising_edge(A(16)) then
--       if TIME_EN = '1' then
--         A_TIME(16) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A17_TIME: process(nLBRES, A(17), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(17) <= (others => '0');
--     elsif rising_edge(A(17)) then
--       if TIME_EN = '1' then
--         A_TIME(17) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A18_TIME: process(nLBRES, A(18), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(18) <= (others => '0');
--     elsif rising_edge(A(18)) then
--       if TIME_EN = '1' then
--         A_TIME(18) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A19_TIME: process(nLBRES, A(19), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(19) <= (others => '0');
--     elsif rising_edge(A(19)) then
--       if TIME_EN = '1' then
--         A_TIME(19) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A20_TIME: process(nLBRES, A(20), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(20) <= (others => '0');
--     elsif rising_edge(A(20)) then
--       if TIME_EN = '1' then
--         A_TIME(20) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A21_TIME: process(nLBRES, A(21), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(21) <= (others => '0');
--     elsif rising_edge(A(21)) then
--       if TIME_EN = '1' then
--         A_TIME(21) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A22_TIME: process(nLBRES, A(22), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(22) <= (others => '0');
--     elsif rising_edge(A(22)) then
--       if TIME_EN = '1' then
--         A_TIME(22) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A23_TIME: process(nLBRES, A(23), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(23) <= (others => '0');
--     elsif rising_edge(A(23)) then
--       if TIME_EN = '1' then
--         A_TIME(23) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A24_TIME: process(nLBRES, A(24), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(24) <= (others => '0');
--     elsif rising_edge(A(24)) then
--       if TIME_EN = '1' then
--         A_TIME(24) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A25_TIME: process(nLBRES, A(25), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(25) <= (others => '0');
--     elsif rising_edge(A(25)) then
--       if TIME_EN = '1' then
--         A_TIME(25) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A26_TIME: process(nLBRES, A(26), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(26) <= (others => '0');
--     elsif rising_edge(A(26)) then
--       if TIME_EN = '1' then
--         A_TIME(26) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A27_TIME: process(nLBRES, A(27), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(27) <= (others => '0');
--     elsif rising_edge(A(27)) then
--       if TIME_EN = '1' then
--         A_TIME(27) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A28_TIME: process(nLBRES, A(28), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(28) <= (others => '0');
--     elsif rising_edge(A(28)) then
--       if TIME_EN = '1' then
--         A_TIME(28) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A29_TIME: process(nLBRES, A(29), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(29) <= (others => '0');
--     elsif rising_edge(A(29)) then
--       if TIME_EN = '1' then
--         A_TIME(29) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A30_TIME: process(nLBRES, A(30), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(30) <= (others => '0');
--     elsif rising_edge(A(30)) then
--       if TIME_EN = '1' then
--         A_TIME(30) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_A31_TIME: process(nLBRES, A(31), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       A_TIME(31) <= (others => '0');
--     elsif rising_edge(A(31)) then
--       if TIME_EN = '1' then
--         A_TIME(31) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;

--   P_B0_TIME: process(nLBRES, B(0), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       B_TIME(0) <= (others => '0');
--     elsif rising_edge(B(0)) then
--       if TIME_EN = '1' then
--         B_TIME(0) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_B1_TIME: process(nLBRES, B(1), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       B_TIME(1) <= (others => '0');
--     elsif rising_edge(B(1)) then
--       if TIME_EN = '1' then
--         B_TIME(1) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_B2_TIME: process(nLBRES, B(2), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       B_TIME(2) <= (others => '0');
--     elsif rising_edge(B(2)) then
--       if TIME_EN = '1' then
--         B_TIME(2) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_B3_TIME: process(nLBRES, B(3), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       B_TIME(3) <= (others => '0');
--     elsif rising_edge(B(3)) then
--       if TIME_EN = '1' then
--         B_TIME(3) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;
--
--   P_B31_TIME: process(nLBRES, B(31), EXT_COINC_RES)
--   begin
--     if nLBRES = '0' or EXT_COINC_RES = '0' then
--       B_TIME(31) <= (others => '0');
--     elsif rising_edge(B(31)) then
--       if TIME_EN = '1' then
--         B_TIME(31) <= COUNTER(31 downto 0);
--       end if;
--     end if;
--   end process;


--   --**********************************************************
--   -- Select delay timer
--   --**********************************************************
--   P_DLY_SEL: process(DELAY_SEL, STARTDELAY,  
--                      PDL_IN_i,  PDL0_OUT, PDL1_OUT,
--                      DLO0_OUT,  DLO1_OUT)
--   begin
--     PDL0_IN       <= '0';
--     PDL1_IN       <= '0';
--     DLO0_GATE     <= '0';
--     DLO1_GATE     <= '0';
--     DLO_PULSE     <= '0';
--     PDL_PULSEOUT  <= '0';
--    case DELAY_SEL is
--       when "00" =>   PDL0_IN       <= PDL_IN_i;
--                      PDL_PULSEOUT  <= PDL_IN_i and (not PDL0_OUT);
--       when "01" =>   PDL1_IN       <= PDL_IN_i;
--                      PDL_PULSEOUT  <= PDL_IN_i and (not PDL1_OUT);
--       when "10" =>   DLO0_GATE     <= STARTDELAY;
--                      DLO_PULSE     <= DLO0_OUT;
--       when "11" =>   DLO1_GATE     <= STARTDELAY;
--                      DLO_PULSE     <= DLO1_OUT;
--       when others => null;
--     end case;
--   end process;    
--
--   -- DDL WAVEFORM GENERATOR
--   -- Monolithic DDL (3D3428) has a minimum pulse width specs (320 ns)
--   -- for optimum linearity performance.
--   --
--   --
--   --           _____________________         
--   --   _______|                     |_________________
--   --            _____________________            
--   --   ________|                     |_________________
--   --           _            
--   --   _______||_______________________________________
--   
--   process(nLBRES, LCLK)
--   begin
--     if nLBRES = '0' then
--       PDL_IN_i <= '0';
--       WVF_CNT  <= (others => '0');
--       ENABLE_CNT <= '0';
--     elsif LCLK'event and LCLK='1' then 
--       if STARTDELAY = '1' then
--         PDL_IN_i   <= '1';
--         ENABLE_CNT <= '1';
--       end if;
--       if ENABLE_CNT = '1' then
--          WVF_CNT <= WVF_CNT + 1;
--       end if;
--       if WVF_CNT = "01101" then
--         PDL_IN_i <= '0';
--         WVF_CNT  <= (others => '0'); 
--         ENABLE_CNT <= '0';
--       end if;
--     end if;
--   end process;
--
--
--  --****************************************
--  -- DLO DELAY COUNTER
--  --****************************************
--  -- Counts pulses out of the selected delay 
--  -- line before closing the GATE signal
--  P_DLO_DELAY: process(nLBRES,DLO_PULSE, STARTDELAY)
--  begin
--    if ((nLBRES = '0') or (STARTDELAY = '0')) then
--       CNT          <= (others => '0');
--       STOP_DLO     <= '0';
--       DLO_PULSEOUT <= '0';
--    elsif DLO_PULSE'event and DLO_PULSE = '1' then
--         CNT          <= CNT + 1;
--         DLO_PULSEOUT <= '1';
--         if CNT = GATEWIDTH then
--            STOP_DLO     <= '1';
--            DLO_PULSEOUT <= '0';
--         end if;
--       end if;
--  end process;
--  
--  --****************************************
--  -- PDL SINGLE SHOT ACQUISITION
--  --****************************************
--  -- This process detects the PDL pulse and resets
--  -- the Coincidence register to allow a new coincidence
--  -- to be detected.
--  P_PDL_DELAY: process(nLBRES,PDL_PULSEOUT, STARTDELAY)
--  begin
--    if ((nLBRES = '0') or (STARTDELAY = '0')) then
--       STOP_PDL  <= '0';
--    elsif PDL_PULSEOUT'event and PDL_PULSEOUT = '0' then
--       STOP_PDL  <= '1';
--    end if;
--  end process;
-- 
--  --********************************************************************************
--  -- DLOx: DELAY LINE OSCILLATORS
--  --   GATE DRIVER
--  --********************************************************************************
--
--   --********************************************************************************
--  -- PULSE MODE = '0' (GATEWIDTH = 3):
--  --   EXT_GATE leading edge synchronous with PULSE leading egde.
--  --               ____
--  -- COINC     ___|    |________________________________________
--  --                  ___     ___     ___     _
--  -- DLO_PULSE ______|   |___|   |___|   |___| |________________
--  --                  _______________________
--  -- DLO_GATE  ______|                       |__________________
--  --
--   --********************************************************************************
--  -- PULSE MODE = '1' (GATEWIDTH = 3):
--  --   EXT_GATE leading edge synchronous with COINC leading egde.
--  --               ____
--  -- COINC     ___|    |________________________________________
--  --                  ___     ___     ___     _
--  -- DLO_PULSE ______|   |___|   |___|   |___| |________________
--  --               __________________________
--  -- DLO_GATE  ___|                          |__________________
--  --
--  DLO_GATE <= DLO_PULSEOUT when PULSE_MODE = '0' else
--              STARTDELAY;
--
--  --********************************************************************************
--  -- PDLx: PROGRAMMABLE DELAY LINES
--  --   GATE DRIVER
--  --   NOTE: GATEWIDTH HAS NO EFFECT IN CASE OF A PDL-BASED GATE PULSE.
--  --   THE GATE PULSE WIDTH DEPENDS ON CURRENT DELAY OF THE PDL CHIP.
--  --********************************************************************************
--
--   --********************************************************************************
--  -- PULSE MODE = '0' :
--  --   EXT_GATE leading edge synchronous with PULSE leading egde.
--  --                  ____
--  -- COINC        ___|    |________________________________________
--  --                     ___ 
--  -- PDL_PULSEOUT ______|   |______________________________________
--  --                     ___
--  -- PDL_GATE     ______|   |______________________________________
--  --
--   --********************************************************************************
--  -- PULSE MODE = '1' :
--  --   EXT_GATE leading edge synchronous with COINC leading egde.
--  --                  ____
--  -- COINC        ___|    |________________________________________
--  --                     ___ 
--  -- PDL_PULSEOUT ______|   |______________________________________
--  --                  ______
--  -- PDL_GATE     ___|      |______________________________________
--  --              
--  PDL_GATE <= PDL_PULSEOUT    when PULSE_MODE = '0' else
--              STARTDELAY;
--  
--
--   -- GATE PULSE
--   process(DELAY_SEL, PDL_GATE, DLO_GATE)
--   begin
--       case DELAY_SEL is
--         when "00" | "01" =>    EXT_GATE  <= PDL_GATE;         -- PDL0, PDL1
--         when "10" | "11" =>    EXT_GATE  <= DLO_GATE;         -- DLO0, DLO1                   
--         when others => null;
--       end case;
--   end process;      

   --********************************************************************************
   -- USER REGISTERS SECTION
   --********************************************************************************


   -- WRITE REGISTERS
   P_WREG : process(LCLK, nLBRES)
   begin
      if (nLBRES = '0') then
         A_MASK       <= X"FFFFFFFF"; -- Default : Unmasked
         B_MASK       <= X"FFFFFFFF"; -- Default : Unmasked
         C_MASK       <= X"FFFFFFFF"; -- Default : Unmasked
         GATEWIDTH    <= X"00C8";     -- Default : 1 us
         C_CONTROL    <= X"00000000";
         MODE         <= X"0000";     -- Default : I/O Register
         SCRATCH      <= X"5A5A";
         G_CONTROL    <= X"00000000"; -- Default : Enable G port / Level=TTL
         D_CONTROL    <= X"00000000"; -- Default : Enable D port / Level=TTL
         D_DATA       <= X"00000000"; 
         E_CONTROL    <= X"00000000"; -- Default : Enable E port / Level=TTL
         E_DATA       <= X"00000000"; 
         F_CONTROL    <= X"00000000"; -- Default : Enable F port / Level=TTL
         F_DATA       <= X"00000000";
         PDL_CONTROL  <= X"0001";     -- DLY Buffer DIR = DLY -> FPGA; PDL Write Enabled
         PDL_DATA     <= X"0000";
         -- Custom Registers
         CSCRATCH      <= X"DEAD";
         CUSTOM_CNTRL  <= X"0000";
         COINC_RES_REG <= X"FFFF";
         SI_GATEWIDTH  <= X"0028";
         SI_RES_DELAY  <= X"0028";
         SI_DELAY(0)   <= X"8000";
         SI_DELAY(1)    <= X"8000";
         SI_DELAY(2)    <= X"8000";
         SI_DELAY(3)    <= X"8000";
         BGO_DELAY_T_SHORT <= X"5850";
         BGO_DELAY_T_LONG <= X"5820";
         BLOCK_BGO_DELAY<= X"0100";
         COINC_MASK    <= X"FFFF";
       elsif LCLK'event and LCLK = '1' then
         if (REG_WREN = '1') and (USR_ACCESS = '1') then
           case REG_ADDR is
             when A_AMASK_L   => A_MASK(15 downto 0)      <= REG_DIN;
             when A_AMASK_H   => A_MASK(31 downto 16)     <= REG_DIN;
             when A_BMASK_L   => B_MASK(15 downto 0)      <= REG_DIN;
             when A_BMASK_H   => B_MASK(31 downto 16)     <= REG_DIN;
             when A_CMASK_L   => C_MASK(15 downto 0)      <= REG_DIN;
             when A_CMASK_H   => C_MASK(31 downto 16)     <= REG_DIN;
             when A_GATEWIDTH => GATEWIDTH                <= REG_DIN;
             when A_CCTRL_L   => C_CONTROL(15 downto 0)   <= REG_DIN;
             when A_CCTRL_H   => C_CONTROL(31 downto 16)  <= REG_DIN;   
             when A_MODE      => MODE                     <= REG_DIN;
             when A_SCRATCH   => SCRATCH                  <= REG_DIN; 
             when A_GCTRL     => G_CONTROL(15 downto 0)   <= REG_DIN; 
             when A_DCTRL_L   => D_CONTROL(15 downto  0)  <= REG_DIN;
             when A_DCTRL_H   => D_CONTROL(31 downto 16)  <= REG_DIN;
             when A_DDATA_L   => D_DATA   (15 downto  0)  <= REG_DIN;
             when A_DDATA_H   => D_DATA   (31 downto 16)  <= REG_DIN;
             when A_ECTRL_L   => E_CONTROL(15 downto  0)  <= REG_DIN;
             when A_ECTRL_H   => E_CONTROL(31 downto 16)  <= REG_DIN;
             when A_EDATA_L   => E_DATA   (15 downto  0)  <= REG_DIN;
             when A_EDATA_H   => E_DATA   (31 downto 16)  <= REG_DIN;
             when A_FCTRL_L   => F_CONTROL(15 downto  0)  <= REG_DIN;
             when A_FCTRL_H   => F_CONTROL(31 downto 16)  <= REG_DIN;
             when A_FDATA_L   => F_DATA   (15 downto  0)  <= REG_DIN;
             when A_FDATA_H   => F_DATA   (31 downto 16)  <= REG_DIN;
             when A_PDL_CTRL  => PDL_CONTROL              <= REG_DIN;
             when A_PDL_DATA  => PDL_DATA                 <= REG_DIN;
             -- Custom Registers
             when A_CSCRATCH     => CSCRATCH              <= REG_DIN;
             when A_CUSTOM_CNTRL => CUSTOM_CNTRL          <= REG_DIN;
             when A_COINC_RESET  => COINC_RES_REG         <= REG_DIN;
             when A_SIGATEWIDTH  => SI_GATEWIDTH          <= REG_DIN;
             when A_SIRESDELAY   => SI_RES_DELAY          <= REG_DIN;
             when A_SIDELAY_0    => SI_DELAY(0)           <= REG_DIN;
             when A_SIDELAY_1    => SI_DELAY(1)           <= REG_DIN;
             when A_SIDELAY_2    => SI_DELAY(2)           <= REG_DIN;
             when A_SIDELAY_3    => SI_DELAY(3)           <= REG_DIN;
             when A_BGODELAY_SHORT=>BGO_DELAY_T_SHORT     <= REG_DIN;
             when A_BGODELAY_LONG=> BGO_DELAY_T_LONG      <= REG_DIN;
             when A_BGOBLOCKDELAY=> BLOCK_BGO_DELAY       <= REG_DIN;
             when A_COINCMASK_0  => COINC_MASK            <= REG_DIN;
             when others         => null;
           end case;
         end if;
       end if;
     end process;
   
     
  -- READ REGISTERS
  P_RREG: process(LCLK, nLBRES)
  begin
       if (nLBRES = '0') then
         REG_DOUT   <= (others => '0');
       elsif LCLK'event and LCLK = '1' then
         if (REG_RDEN = '1') and (USR_ACCESS = '1') then
           case REG_ADDR is
             when A_ASTATUS_L   => REG_DOUT   <= A_STATUS (15 downto 0);
             when A_ASTATUS_H   => REG_DOUT   <= A_STATUS (31 downto 16);
             when A_BSTATUS_L   => REG_DOUT   <= B_STATUS (15 downto 0);
             when A_BSTATUS_H   => REG_DOUT   <= B_STATUS (31 downto 16);
             when A_CSTATUS_L   => REG_DOUT   <= C_STATUS (15 downto 0);
             when A_CSTATUS_H   => REG_DOUT   <= C_STATUS (31 downto 16);
             when A_SCRATCH     => REG_DOUT   <= SCRATCH; 
             when A_DCTRL_L     => REG_DOUT   <= D_CONTROL(15 downto  0);
             when A_DCTRL_H     => REG_DOUT   <= D_CONTROL(31 downto 16);
             when A_DDATA_L     => REG_DOUT   <= D_DIN    (15 downto  0);
             when A_DDATA_H     => REG_DOUT   <= D_DIN    (31 downto 16);
             when A_ECTRL_L     => REG_DOUT   <= E_CONTROL(15 downto  0);
             when A_ECTRL_H     => REG_DOUT   <= E_CONTROL(31 downto 16);
             when A_EDATA_L     => REG_DOUT   <= E_DIN    (15 downto  0);
             when A_EDATA_H     => REG_DOUT   <= E_DIN    (31 downto 16);
             when A_FCTRL_L     => REG_DOUT   <= F_CONTROL(15 downto  0);
             when A_FCTRL_H     => REG_DOUT   <= F_CONTROL(31 downto 16);
             when A_FDATA_L     => REG_DOUT   <= F_DIN    (15 downto  0);
             when A_FDATA_H     => REG_DOUT   <= F_DIN    (31 downto 16);
             when A_REVISION    => REG_DOUT   <= REVISION;
             when A_PDL_CTRL    => REG_DOUT   <= PDL_CONTROL;
             when A_PDL_DATA    => REG_DOUT   <= X"00"  & PDL_READ;
             when A_DIDCODE     => REG_DOUT   <= X"000" & '0' & D_IDCODE;
             when A_EIDCODE     => REG_DOUT   <= X"000" & '0' & E_IDCODE;
             when A_FIDCODE     => REG_DOUT   <= X"000" & '0' & F_IDCODE;
             -- Custom Registers
             when A_CREVISION     => REG_DOUT <= CREVISION;
             when A_CSCRATCH      => REG_DOUT <= CSCRATCH;
             when A_CUSTOM_STATUS => REG_DOUT <= CUSTOM_STATUS;
             when A_ATRIGREG_L    => REG_DOUT <= ATRIG_REG(15 downto 0);
             when A_ATRIGREG_H    => REG_DOUT <= ATRIG_REG(31 downto 16);
--             when A_BTRIGREG_L    => REG_DOUT <= BTRIG_REG(15 downto 0);
--             when A_BTRIGREG_H    => REG_DOUT <= BTRIG_REG(31 downto 16);
             when A_COINCREG_0    => REG_DOUT <= COINC_ARRAY(15 downto 0);
             when A_COUNTERREG_0  => REG_DOUT <= COUNTER_REG(15 downto 0);
             when A_COUNTERREG_1  => REG_DOUT <= COUNTER_REG(31 downto 16);
             when A_COUNTERREG_2  => REG_DOUT <= COUNTER_REG(47 downto 32);
             when A_COUNTERREG_3  => REG_DOUT <= COUNTER_REG(63 downto 48);
             when others          => REG_DOUT <= (others => '0');
           end case;
           -- Custom Register Loop
           for i in 0 to 31 loop
             case REG_ADDR is
               when A_ATIME_L(i) => REG_DOUT <= A_TIME(i)(15 downto  0);
               when A_ATIME_H(i) => REG_DOUT <= A_TIME(i)(31 downto 16);
               when A_BTIME_L(i) => REG_DOUT <= B_TIME(i)(15 downto  0);
               when A_BTIME_H(i) => REG_DOUT <= B_TIME(i)(31 downto 16);
               when others            => null;
             end case;
           end loop;
         end if;
       end if;
     end process;


END rtl;

